<?php
include_once dirname( __FILE__ ) . '/class-elementorsheets-setting.php';
use ElementorPro\Plugin;
use Elementor\Controls_Manager;
use ElementorPro\Modules\Forms\Classes\Action_Base;
use ElementorPro\Modules\Forms\Controls\Fields_Map;
/**
 * Class Elementor_Form_Sheets_Action
 * @see https://developers.elementor.com/custom-form-action/
 * Custom elementor form action after submit to add a records to
 * Google Spreadsheet
 */
class Elementor_Form_Sheets_Action extends \ElementorPro\Modules\Forms\Classes\Action_Base {
	/**
	 * Get Name
	 *
	 * Return the action name
	 *
	 * @access public
	 * @return string
	 */
	public function get_name() {
		return esc_html__( 'elementorsheets', 'elementorsheets' );
	}
	/**
	 * Get Label
	 *
	 * Returns the action label
	 *
	 * @access public
	 * @return string
	 */
	public function get_label() {
		return esc_html__( 'ElementorSheets', 'elementorsheets' );
	}
	/**
	 * Run
	 *
	 * Runs the action after submit
	 *
	 * @access public
	 * @param \ElementorPro\Modules\Forms\Classes\Form_Record $record
	 * @param \ElementorPro\Modules\Forms\Classes\Ajax_Handler $ajax_handler
	 */
	public function run( $record, $ajax_handler ) {
		
		
		$elementorsheets_settings = $record->get( 'form_settings' );
		// Get sumitetd Form data
		$elementorsheets_raw_fields = $record->get( 'fields' );
		
		if( isset($elementorsheets_settings['submit_actions']) && in_array( $this->get_name() ,$elementorsheets_settings['submit_actions'])){
			
			$elementorsheets_client 				= ElementorSheets_Setting::elementorsheets_getClient();
			$elementorsheets_spreadsheetid 			= $elementorsheets_settings['spreadsheetid'];
			$elementorsheets_sheetname 				= $elementorsheets_settings['sheet_name'];
			$elementorsheets_lead_created_date		= '';
			if( isset($elementorsheets_settings['lead_created_date']) )
				$elementorsheets_lead_created_date 	= $elementorsheets_settings['lead_created_date'];
			
			$elementorsheets_page_url		= '';
			if( isset($elementorsheets_settings['lead_created_page_url']) )
				$elementorsheets_page_url 	= $elementorsheets_settings['lead_created_page_url'];
			
		
			
			if( empty( $elementorsheets_spreadsheetid ) || empty( $elementorsheets_sheetname ) )
				return;
			// Normalize the Form Data
			
			$elementorsheets_fields = array();
			foreach ( $elementorsheets_raw_fields as $id => $field ) {
				$elementorsheets_fields[ $id ] = $field['value'];
			}
			
			try{
				$elementorsheets_client 		= ElementorSheets_Setting::elementorsheets_getClient();
				$elementorsheets_service 		= new Google_Service_Sheets($elementorsheets_client);
				
				if( isset($elementorsheets_settings['elementorsheets_fields_map']) && !empty($elementorsheets_settings['elementorsheets_fields_map']) ){				
					$elementorsheets_headers 		= array_column( $elementorsheets_settings['elementorsheets_fields_map'], 'local_id' );
				}else{
					$elementorsheets_headers 		= $elementorsheets_settings['sheet_headers'];		
				}
				
				
				$elementorsheets_value_data = array();
				foreach( $elementorsheets_headers as $elementorsheets_fieldvalue ){
					if( array_key_exists($elementorsheets_fieldvalue,$elementorsheets_fields) ){
						
						if( is_array( $elementorsheets_fields[$elementorsheets_fieldvalue] ) )
							$elementorsheets_value_data[] = implode(',', $elementorsheets_fields[$elementorsheets_fieldvalue]);
						else
							$elementorsheets_value_data[] = $elementorsheets_fields[$elementorsheets_fieldvalue];	
							
					}else{
						$elementorsheets_value_data[] = '';
					}	
				}
				
				$elementorsheets_sheet 		= "'".$elementorsheets_sheetname."'!A:A";
				$elementorsheets_allentry 	= $elementorsheets_service->spreadsheets_values->get($elementorsheets_spreadsheetid, $elementorsheets_sheet);
				$elementorsheets_data 		= $elementorsheets_allentry->getValues();
				
				$elementorsheets_data = array_map(
					function($elementorsheets_element)
					{
						if(isset($elementorsheets_element['0'])){
							return $elementorsheets_element['0'];
						}else{
							return "";
						}
					}, 
					$elementorsheets_data
				);
				
				if( $elementorsheets_lead_created_date == 'yes' ){
					$elementorsheets_value_data[] = date("F j, Y, g:i A", current_time( 'timestamp', 0 ));
				}
				
				if(  $elementorsheets_page_url == 'yes' ){
					if( isset( $_SERVER['HTTP_REFERER'] ) ){
						$elementorsheets_value_data[] = $_SERVER['HTTP_REFERER'];	
					}
				}
					
				$elementorsheets_rangetoupdate 	= $elementorsheets_sheetname.'!A'.( count($elementorsheets_data) + 1 );
				$elementorsheets_requestBody 	= new Google_Service_Sheets_ValueRange( array( 'values' => array($elementorsheets_value_data )) );
				$elementorsheets_params 		= array( 'valueInputOption' => 'RAW' ); 
				$elementorsheets_response 		= $elementorsheets_service->spreadsheets_values->update( $elementorsheets_spreadsheetid, $elementorsheets_rangetoupdate, $elementorsheets_requestBody, $elementorsheets_params );	
			}
			//catch exception
			catch(Exception $e) {
			  	$ajax_handler->add_admin_error_message( 'ElementorSheets ' . $exception->getMessage() );
			}
		}
	}
		
	/**
	 * Register Settings Section
	 *
	 * Registers the Action controls
	 *
	 * @access public
	 * @param \Elementor\Widget_Base $widget
	 */
	public function register_settings_section( $widget ) {
		
		$elementorsheets_google_settings = get_option('elementorsheets_google_settings');
		
		if( ( !empty($elementorsheets_google_settings[2])  ) ){
			ElementorSheets_Setting::elementorsheets_getClient();
		}
		//$widget->get_settings('elementorsheets_fields_map');
		
		
		
		global $elementorsheets_headers, $elementorsheets_default_headers, $elementorsheets_exclude_headertype;
		global $elementorsheets_spreadsheetid , $elementorsheets_sheetname, $elementorsheets_sheet_headers, $elementorsheets_sheetheaders;
		
		$elementorsheets_exclude_headertype 	= array( 'honeypot', 'recaptcha', 'recaptcha_v3', 'html');
		$elementorsheets_document 				= Plugin::elementor()->documents->get( get_the_ID() );
		
		if ( $elementorsheets_document ) {
			
			$elementorsheets_data = $elementorsheets_data_global = $elementorsheets_document->get_elements_data();
			$elementorsheets_data = Plugin::elementor()->db->iterate_data( $elementorsheets_data, function( $element ) use ( &$do_update ) {
				if ( isset($element['widgetType']) && 'form' == $element['widgetType'] ) {
					
					global $elementorsheets_headers, $elementorsheets_default_headers, $elementorsheets_exclude_headertype;
					global $elementorsheets_spreadsheetid , $elementorsheets_sheetname, $elementorsheets_sheet_headers,$elementorsheets_sheetheaders;
				
					if( isset($element['settings']['spreadsheetid']) )
						$elementorsheets_spreadsheetid 	= $element['settings']['spreadsheetid'];
						
					if( isset($element['settings']['sheet_name']) )
						$elementorsheets_sheetname 	= $element['settings']['sheet_name'];
						
					if( isset($element['settings']['elementorsheets_fields_map']) ){
						$elementorsheets_sheetheaders = array_column( $element['settings']['elementorsheets_fields_map'], 'remote_label');	
					}
						//$elementorsheets_sheet_headers 	= $element['settings']['sheet_headers'];
					
					foreach ( $element['settings']['form_fields'] as $formdata ) {	
						
						if( !in_array( $formdata['field_type'], $elementorsheets_exclude_headertype ) )
							$elementorsheets_headers[$formdata['custom_id']] 	= $formdata['field_label']?$formdata['field_label']:ucfirst($formdata['custom_id']);
							$elementorsheets_default_headers[] 					= $formdata['custom_id'];
					}
					//return $elementorsheets_headers;
					
					return $elementorsheets_sheetheaders;
				}
			} );
			
			if( empty($elementorsheets_headers) ){
				
				Plugin::elementor()->db->iterate_data( $elementorsheets_data_global, function( $element ) use ( &$do_update ) {
				
				if ( isset($element['widgetType']) && 'global' == $element['widgetType'] ) {
					if ( ! empty( $element['templateID'] ) ) {
						
						$global_form 		= get_post_meta( $element['templateID'], '_elementor_data', true );
						$global_form_meta 	= json_decode( $global_form,true);
						if ( $global_form_meta ) {
							global $elementorsheets_headers, $elementorsheets_default_headers, $elementorsheets_exclude_headertype;
							global $elementorsheets_spreadsheetid , $elementorsheets_sheetname, $elementorsheets_sheet_headers;
							
							
							if( isset( $global_form_meta[0]['settings']['spreadsheetid'] ) )
								$elementorsheets_spreadsheetid 	= $global_form_meta[0]['settings']['spreadsheetid'];
								
							if( isset( $global_form_meta[0]['settings']['sheet_name'] ) )
								$elementorsheets_sheetname 	= $global_form_meta[0]['settings']['sheet_name'];
								
							if( isset( $global_form_meta[0]['settings']['sheet_headers'] ) )
								$elementorsheets_sheet_headers 	= $global_form_meta[0]['settings']['sheet_headers'];
							
							if( is_array($global_form_meta[0]['settings']['form_fields']) ) {
								foreach ( $global_form_meta[0]['settings']['form_fields'] as $formdata ) {	
									
									if( !in_array( $formdata['field_type'], $elementorsheets_exclude_headertype ) )
										$elementorsheets_headers[$formdata['custom_id']] 	= $formdata['field_label']?$formdata['field_label']:ucfirst($formdata['custom_id']);
										$elementorsheets_default_headers[] 					= $formdata['custom_id'];
								}
							}
							return $elementorsheets_headers;	
						}
					}
					
				}
			} );	
			}
		}
		if( !is_array($elementorsheets_sheetheaders) ){
			$elementorsheets_sheetheaders = array();
		}
		
		if( empty($elementorsheets_google_settings[2] ) ){
			
			$elementorsheets_html = sprintf(
				'<div class="elementor-control-raw-html elementor-panel-alert elementor-panel-alert-danger">%1$s<a href="admin.php?page=elementor_sheets_settings"> <strong>%2$s</strong></a>.</div>',
				esc_html__('Please genearate authentication code from Google Sheet Setting', 'elementorsheets' ),
				esc_html__( 'Click Here', 'elementorsheets' )
			);
						
			$widget->start_controls_section(
				'section_notice_elementorsheets',
				[
					'label' 	=> esc_attr__( 'ElementorSheets', 'elementorsheets' ),
					'condition' => [
						'submit_actions' => $this->get_name(),
					],
				]
			);
			
			$widget->add_control(
				'setup_clientidsecret',
				[
					'type' 	=> \Elementor\Controls_Manager::RAW_HTML,
					'raw' 	=> $elementorsheets_html
				]
			);
			
			$widget->end_controls_section();
			
		}else{
			if( !empty($elementorsheets_headers) && is_array($elementorsheets_headers) ){
				$elementorsheets_headers = implode(',',$elementorsheets_headers);
			}else{
				$elementorsheets_headers = '';	
			}
			$sorting_headers = '<div class="elementorsheets-sheetsheaders"><label class="elementorsheets-sheetsheaders-title">Sheet Headers</label><ul id="sortable" class="ui-sortable">';
			foreach( $elementorsheets_sheetheaders as $header ){
				$newheader = str_replace(' ','_',strtolower($header));
				$sorting_headers .= '<li class="ui-state-default ui-sortable-handle"><label for='.$newheader.'><span class="ui-icon ui-icon-caret-2-n-s"></span>
			  <div id="elementorsheets-panel-field-elementorsheets-sheetheaders-'.$newheader.'-wrap" class="elementorsheets-panel-field custom-swap custom-sort-headers elementorsheets-panel-field-checkbox">
				<label for="elementorsheets-panel-field-elementorsheets-sheetheaders-'.$newheader.'" class="inline hdrtext">'.$header.'</label>
				<label for="elementorsheets-panel-field-elementorsheets-sheetheaders-'.$newheader.'"><label for="elementorsheets-panel-field-elementorsheets-sheetheaders-'.$newheader.'">
				  <span class="ui-icon ui-icon-close"></span>
				  </label>
				<span class="checkbox-switch"></span>
				</label>
			  </div>
			</li>';		
			}
       		 $sorting_headers .= '</ul></div><div id="form_all_headers" style="display:none">'.$elementorsheets_headers.'</div>';
			
			
			$elementorsheets_spreadsheets = ElementorSheets_Setting::elementorsheets_list_googlespreedsheet();
			
			$widget->start_controls_section(
				'section_elementorsheets',
				[
					'label' 	=> esc_attr__( 'ElementorSheets', 'elementorsheets' ),
					'condition' => [
						'submit_actions' => $this->get_name(),
						],
				]
			);
			
			$widget->add_control(
				'spreadsheetid',
				[
					'label'		  	=>  esc_attr__( 'Select Spreadsheet', 'elementorsheets' ),
					'type' 		  	=>  \Elementor\Controls_Manager::SELECT,
					'options' 	  	=>  $elementorsheets_spreadsheets,
					'label_block' 	=> true,
					'separator'   	=>  'before'
				]
			);
			
			$widget->add_control(
				'new_spreadsheet_name',
				[
					'label' 		=>  esc_attr__( 'Spreadsheet Name', 'elementorsheets' ),
					'type' 			=>  \Elementor\Controls_Manager::TEXT,
					'label_block' 	=> true,
					'condition'    	=> [
						'spreadsheetid' => 'new',
					]
				]
			);
			
			$widget->add_control(
				'sheet_name',
				[
					'label' 		=>  esc_attr__( 'Sheet Name', 'elementorsheets' ),
					'type' 			=>  \Elementor\Controls_Manager::TEXT,
					'label_block' 	=> true
				]
			);
			$widget->add_control(
                'headers_listing_ul',
                [
                    'type' => \Elementor\Controls_Manager::RAW_HTML,
                    'raw'  => $sorting_headers,
					'separator' => 'before'
                ]
            );
			 $widget->add_control(
                'header_name',
                [
                    'label'       => __('Header Name', 'elementorsheets'),
                    'type'        =>  \Elementor\Controls_Manager::TEXT,
                    'label_block' => true,
					'separator' => 'before',
                ]
            );
			$widget->add_control(
				'add_header_title',
				[
					'label' => __( '', 'elementorsheets' ),
					'type' => \Elementor\Controls_Manager::BUTTON,
					'button_type' => 'addcustomheaders',
					'text' => __( 'Add', 'elementorsheets' ),
					'event' => 'namespace:editor:getheaders',
				]
			);
			
			$widget->add_control(
                'elementorsheets_fields_map',
                [
                    'label'     => __('Sheet Headers Mapping Form Fields', 'elementorsheets'),
                    'type'      => Fields_Map::CONTROL_TYPE,
                    'separator' => 'before',
                    'fields'    => [
                        [
                            'name' => 'remote_id',
                            'type' => Controls_Manager::HIDDEN,
                        ],
                        [
                            'name' => 'local_id',
                            'type' => Controls_Manager::SELECT,
                        ],
                    ]
                ]
            );	
			
			$widget->add_control( 
				'lead_created_date', 
				[
					'label'        => esc_attr__( 'Submission Date', 'elementorsheets' ),
					'type'         => \Elementor\Controls_Manager::SWITCHER,
					'label_off'    => 'No',
					'label_on'     => 'Yes',
					'return_value' => 'yes'
				] 
			);
			
			$widget->add_control( 
				'lead_created_page_url', 
				[
					'label'        => esc_attr__( 'Page Referer Url', 'elementorsheets' ),
					'type'         => \Elementor\Controls_Manager::SWITCHER,
					'label_off'    => 'No',
					'label_on'     => 'Yes',
					'return_value' => 'yes'
				] 
			);
			
			$widget->add_control( 
				'freeze_header', 
				[
					'label'        => esc_attr__( 'Freeze Headers', 'elementorsheets' ),
					'type'         => \Elementor\Controls_Manager::SWITCHER,
					'label_off'    => 'No',
					'label_on'     => 'Yes',
					'return_value' => 'yes'
				] 
			);
			
			$widget->end_controls_section();
		}
	}
	/**
	 * On Export
	 *
	 * Clears form settings on export
	 * @access Public
	 * @param array $element_sheets
	 */
	public function on_export( $element_sheets ) {
	}
}
?>